@@ -84,8 +84,7 @@ sub initialize {
 
 #TODO: <include href="..."> includes a file
 	$self->SUPER::initialize(%options);
-	$self->{options}{'tagsonly'}=1;
-	$self->{options}{'tags'}.='
+	$self->{options}{'_default_translated'}.='
 		w<abstract>
 		<author>
 		<b>
@@ -120,7 +119,7 @@ sub initialize {
 		<var>
 		<version>
 		w<warn>';
-	$self->{options}{'attributes'}.='
+	$self->{options}{'_default_attributes'}.='
 		<author>title
 		<figure>caption
 		<figure>link
@@ -129,7 +128,7 @@ sub initialize {
 		<guide>link
 		<p>by
 		<pre>caption';
-	$self->{options}{'inline'}.='
+	$self->{options}{'_default_inline'}.='
 		<b>
 		<brite>
 		<c>
