@@ -649,38 +649,38 @@ NEW_LINE:
         my ($l, $t, $c) = ($1, $2, $3);
         $line = $l;
         unless ($allow_generated) {
-        # Check for comments indicating that the file was generated.
-        if ($c =~ /Pod::Man/) {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
-            exit 254;
-        } elsif ($c =~ /generated by help2man/)    {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
-        } elsif ($c =~ /with docbook-to-man/)      {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a."));
-            exit 254;
-        } elsif ($c =~ /generated by docbook2man/) {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook2man. Translate the source file with the sgml module of po4a."));
-            exit 254;
-        } elsif ($c =~ /created with latex2man/)   {
-            warn wrap_mod("po4a::man", dgettext("po4a",
-                "This file was generated with %s. ".
-                "You should translate the source file, but continuing anyway."
-                ),"latex2man");
-        } elsif ($c =~ /Generated by db2man.xsl/)  { 
-            warn wrap_mod("po4a::man", dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a."));
-            exit 254;
-        } elsif ($c =~ /generated automatically by mtex2man/)  {
-            warn wrap_mod("po4a::man", dgettext("po4a",
-                "This file was generated with %s. ".
-                "You should translate the source file, but continuing anyway."
-                ),"mtex2man");
-        } elsif ($c =~ /THIS FILE HAS BEEN AUTOMATICALLY GENERATED.  DO NOT EDIT./ ||
-                 $c =~ /DO NOT EDIT/i || $c =~ /generated/i) {
-            warn wrap_mod("po4a::man", dgettext("po4a",
-                "This file contains the line '%s'. ".
-                "You should translate the source file, but continuing anyway."
-                ),$l."\\\"".$c);
-        }
+                # Check for comments indicating that the file was generated.
+                if ($c =~ /Pod::Man/) {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /generated by help2man/)    {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
+                } elsif ($c =~ /with docbook-to-man/)      {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /generated by docbook2man/) {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with docbook2man. Translate the source file with the sgml module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /created with latex2man/)   {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                        "This file was generated with %s. ".
+                        "You should translate the source file, but continuing anyway."
+                        ),"latex2man");
+                } elsif ($c =~ /Generated by db2man.xsl/)  { 
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /generated automatically by mtex2man/)  {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                        "This file was generated with %s. ".
+                        "You should translate the source file, but continuing anyway."
+                        ),"mtex2man");
+                } elsif ($c =~ /THIS FILE HAS BEEN AUTOMATICALLY GENERATED.  DO NOT EDIT./ ||
+                         $c =~ /DO NOT EDIT/i || $c =~ /generated/i) {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                        "This file contains the line '%s'. ".
+                        "You should translate the source file, but continuing anyway."
+                        ),$l."\\\"".$c);
+                }
         }
 
         if ($line =~ m/^[.']*$/) {
@@ -1033,7 +1033,7 @@ sub post_trans {
     }
 
     # There must not be an end of line inside an inline macro
-    $str =~ s/(E<\.[^>]*)\n([^>]*>)/$1$2/gs;
+    $str =~ s/(E<\.[^>]*)\n([^>]*>)/$1 $2/gs;
 
     # No . or ' on first char, or nroff will think it's a macro
     # * at the beginning of a paragraph, add \& (zero width space) at
@@ -1516,92 +1516,26 @@ sub splitargs {
     $arguments =~ s/\\ /$nbs/g;
     $arguments =~ s/^ +//;
     $arguments =~ s/\\&"/\\(dq/g;
-    foreach my $elem (split (/ +/,$arguments)) {
-        print STDERR ">>Seen $elem(buffer=$buffer;esc=$escaped)\n"
-            if ($debug{'splitargs'});
-
-        if (length $buffer && !($elem=~ /\\$/) ) {
-            $buffer .= " ".$elem;
-            print STDERR "Continuation of a quote\n"
-                if ($debug{'splitargs'});
-            # Inside a quoted argument, "" indicates a double quote
-            while ($buffer =~ s/^"([^"]*?)""/"$1\\(dq/) {}
-            # print "buffer=$buffer.\n";
-            if ($buffer =~ m/^"(.*)"(.+)$/) {
-                print STDERR "End of quote, with stuff after it\n"
-                    if ($debug{'splitargs'});
-                my ($a,$b)=($1,$2);
-                $a =~ s/\Q$nbs/\\ /g;
-                $b =~ s/\Q$nbs/\\ /g;
-                push @args,$a;
-                push @args,$b;
-                $buffer = "";
-            } elsif ($buffer =~ m/^"(.*)"$/) {
-                print STDERR "End of a quote\n"
-                    if ($debug{'splitargs'});
-                my $a = $1;
-                $a =~ s/\Q$nbs/\\ /g;
-                push @args,$a;
-                $buffer = "";
-            } elsif ($escaped) {
-                print STDERR "End of an escaped sequence\n"
-                    if ($debug{'splitargs'});
-                unless(length($elem)){
-                    die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
-                        "Escaped space at the end of macro arg. With high probability, it won't do the trick with po4a (because of wrapping). You may want to remove it and use the .nf/.fi groff macro to control the wrapping."));
-                }
-                $buffer =~ s/\Q$nbs/\\ /g;
-                push @args,$buffer;
-                $buffer = "";
-                $escaped = 0;
-            } else {
-                print STDERR "Quotes: Oops\n"
-                    if ($debug{'splitargs'});
-            }
-        } elsif ($elem =~ m/^"(.*?(?<!")(?:"")*)"(?!")(.+)$/) {
-            print STDERR "Quoted: quote without space\n"
-                if ($debug{'splitargs'});
+    $arguments =~ s/^ *//;
+    while (length $arguments) {
+        if ($arguments =~ s/^"((?:[^"]|"")*)"(?!") *//) {
             my $a = $1;
-            my $b = $2;
-            $a =~ s/""/\\(dq/g;
-            $a =~ s/\Q$nbs/\\ /g;
-            $b =~ s/""/\\(dq/g;
-            $b =~ s/\Q$nbs/\\ /g;
+            $a =~ s/""/"/g if defined $a;
             push @args,$a;
-            push @args,$b;
-        } elsif ($elem =~ m/^"(.*)"$/) {
-            print STDERR "Quoted, no space\n"
-                if ($debug{'splitargs'});
+        } elsif ($arguments =~ s/^"((?:[^"]|"")*) *$//) {
+            # Unterminated quote, but this seems to be handled by removing
+            # the trailing spaces and closing the quotes.
             my $a = $1;
-            # Inside a quoted argument, "" indicates a double quote
-            $a =~ s/""/\\(dq/g;
-            $a =~ s/\Q$nbs/\\ /g;
+            $a =~ s/""/"/g if defined $a;
             push @args,$a;
-        } elsif ($elem =~ m/^"/) { #") {
-            print STDERR "Begin of a quoting arg\n"
-                if ($debug{'splitargs'});
-            $buffer=$elem;
-        } elsif ($elem =~ m/^(.*)\\$/) {
-            print STDERR "escaped space after $1\n"
-                if ($debug{'splitargs'});
-            # escaped space
-            $buffer = (length($buffer)?$buffer:'').$1." ";
-            $escaped = 1;
+        } elsif ($arguments =~ s/^([^ ]+) *//) {
+            push @args,$1;
         } else {
-            print STDERR "Unquoted arg, nothing to declare\n"
-                if ($debug{'splitargs'});
-            $elem =~ s/\Q$nbs/\\ /g;
-            push @args,$elem;
-            $buffer="";
+            die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                             "Cannot parse command arguments: %s"),
+                             $arguments)
         }
     }
-    if (length($buffer)) {
-        # Inside a quoted argument, "" indicates a double quote
-        while ($buffer =~ s/^"([^"]*?)""/"$1\\(dq/) {}
-        $buffer=~ s/"//g;
-        $buffer =~ s/\Q$nbs/\\ /g;
-        push @args,$buffer;
-    }
     if ($debug{'splitargs'}) {
         print STDERR "ARGS=";
         map { print STDERR "$_^"} @args;
@@ -2044,8 +1978,8 @@ $macro{'bp'}=\&untranslated;
 $macro{'ad'}=\&untranslated;
 # .de macro Define or redefine macro until .. is encountered.
 $macro{'de'}=sub {
+    my $self = shift;
     if ($groff_code ne "fail") {
-        my $self = shift;
         my $paragraph = "@_";
         my $end = ".";
         if ($paragraph=~/^[.'][\t ]*de[\t ]+([^\t ]+)[\t ]+([^\t ]+)[\t ]$/) {
@@ -2071,7 +2005,7 @@ $macro{'de'}=sub {
                                               "wrap" => 0) );
         }
     } else {
-        die wrap_mod("po4a::man", dgettext("po4a", "This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported."));
+        die wrap_ref_mod($self->{ref}, "po4a::man", dgettext("po4a", "This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported."));
     }
 };
 # .ds stringvar anything
@@ -2478,7 +2412,7 @@ sub define_mdoc_macros {
     foreach (qw(Ad An Ar Cd Cm Dv Er Ev Fa Fd Fn Ic Li Nm Op Ot Pa St Va Vt Xr)) {
         $inline{$_} = 1;
     }
-    # FIXME: some of thes macro introduce a line in bold.
+    # FIXME: some of these macros introduce a line in bold.
     #        Using \fP in these line is not supported.
     #        do_fonts should be called for every inline line
 
