@@ -1,5 +1,5 @@
 # Locale::Po4a::Po -- manipulation of po files
-# $Id: Po.pm,v 1.91 2008-04-02 20:19:07 nekral-guest Exp $
+# $Id: Po.pm,v 1.95 2009-02-28 22:18:39 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -88,6 +88,7 @@ use Carp qw(croak);
 use File::Path; # mkdir before write
 use File::Copy; # move
 use POSIX qw(strftime floor);
+use Time::Local;
 
 use Encode;
 
@@ -123,16 +124,16 @@ sub new {
 }
 
 # Return the numerical timezone (e.g. +0200)
-# The %z format of strftime is not portable.
-# For example, it indicates "2006-10-25 19:36E. Europe Standard Time" on
-# MS Windows.
+# Neither the %z nor the %s formats of strftime are portable:
+# '%s' is not supported on Solaris and '%z' indicates
+# "2006-10-25 19:36E. Europe Standard Time" on MS Windows.
 sub timezone {
     my @g = gmtime();
     my @l = localtime();
 
     my $diff;
-    $diff  = floor(strftime("%s",@l)/60 +0.5);
-    $diff -= floor(strftime("%s",@g)/60 +0.5);
+    $diff  = floor(timelocal(@l)/60 +0.5);
+    $diff -= floor(timelocal(@g)/60 +0.5);
 
     my $h = floor($diff / 60) + $l[8]; # $l[8] indicates if we are currently
                                        # in a daylight saving time zone
@@ -407,7 +408,7 @@ sub write{
                 $output .= format_comment($comment, ". ")
             }
         }
-        $output .= format_comment($self->{po}{$msgid}{'type'}," type: ")
+        $output .= format_comment($self->{po}{$msgid}{'type'},". type: ")
             if    defined($self->{po}{$msgid}{'type'})
                && length ($self->{po}{$msgid}{'type'});
         $output .= format_comment($self->{po}{$msgid}{'reference'},": ")
@@ -657,7 +658,8 @@ sub gettextize {
                          'flags'     => $flags,
                          'type'      => $typeorig,
                          'reference' => $reforig,
-                         'conflict'  => 1)
+                         'conflict'  => 1,
+                         'transref'  => $potrans->{po}{$trans}{'reference'})
             unless (defined($pores->{po}{$orig})
                     and ($pores->{po}{$orig}{'msgstr'} eq $trans))
         # FIXME: maybe we should be smarter about what reference should be
@@ -1163,10 +1165,10 @@ sub push {
 sub push_raw {
     my $self=shift;
     my %entry=@_;
-    my ($msgid,$msgstr,$reference,$comment,$automatic,$flags,$type)=
+    my ($msgid,$msgstr,$reference,$comment,$automatic,$flags,$type,$transref)=
         ($entry{'msgid'},$entry{'msgstr'},
          $entry{'reference'},$entry{'comment'},$entry{'automatic'},
-         $entry{'flags'},$entry{'type'});
+         $entry{'flags'},$entry{'type'},$entry{'transref'});
     my $keep_conflict = $entry{'conflict'};
 
 #    print STDERR "Push_raw\n";
@@ -1216,12 +1218,21 @@ sub push_raw {
                  quote_text($msgstr));
 
             if ($keep_conflict) {
-                $msgstr = "#-#-#-#-#  choice  #-#-#-#-#\\n".
-                          $self->{po}{$msgid}{'msgstr'}."\\n".
-                          "#-#-#-#-#  choice  #-#-#-#-#\\n".
-                          "$msgstr";
-                $msgstr = "#-#-#-#-#  choice  #-#-#-#-#\\n".$msgstr
-                    unless ($msgstr =~ m/^#-#-#-#-#  choice  #-#-#-#-#\\n/s);
+                if ($self->{po}{$msgid}{'msgstr'} =~ m/^#-#-#-#-#  .*  #-#-#-#-#\\n/s) {
+                    $msgstr = $self->{po}{$msgid}{'msgstr'}.
+                              "\\n#-#-#-#-#  $transref  #-#-#-#-#\\n".
+                              $msgstr;
+                } else {
+                    $msgstr = "#-#-#-#-#  ".
+                              $self->{po}{$msgid}{'transref'}.
+                              "  #-#-#-#-#\\n".
+                              $self->{po}{$msgid}{'msgstr'}."\\n".
+                              "#-#-#-#-#  $transref  #-#-#-#-#\\n".
+                              $msgstr;
+                }
+                # Every msgid will have the same list of references.
+                # Only keep the last list.
+                $self->{po}{$msgid}{'reference'} = "";
             } else {
             warn wrap_msg(dgettext("po4a",
                                    "Translations don't match for:\n".
@@ -1235,6 +1246,9 @@ sub push_raw {
             }
         }
     }
+    if (defined $transref) {
+        $self->{po}{$msgid}{'transref'} = $transref;
+    }
     if (defined $reference) {
         if (defined $self->{po}{$msgid}{'reference'}) {
             $self->{po}{$msgid}{'reference'} .= " ".$reference;
@@ -1350,8 +1364,14 @@ set, it will return "CHARSET".
 
 sub get_charset() {
     my $self=shift;
+
     $self->{header} =~ /charset=(.*?)[\s\\]/;
-    return $1;
+
+    if (defined $1) {
+        return $1;
+    } else {
+        return "CHARSET";
+    }
 }
 
 =item set_charset($)
