@@ -1,13 +1,11 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Pod.pm,v 1.8 2004/04/28 09:42:10 mquinson-guest Exp $
-#
-# Copyright 2002 by Martin Quinson <Martin.Quinson@ens-lyon.fr>
+# $Id: Pod.pm,v 1.23 2009-03-23 20:43:50 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
-# under the terms of GPL (see COPYING).
+# under the terms of GPL (see COPYING file).
 #
 # This module converts POD to PO file, so that it becomes possible to 
-# translate POD formated documentation. See gettext documentation for
+# translate POD formatted documentation. See gettext documentation for
 # more info about PO files.
 
 ############################################################################
@@ -25,12 +23,51 @@ use warnings;
 
 require Exporter;
 
-use vars qw($VERSION @ISA);
-$VERSION=$Locale::Po4a::TransTractor::VERSION;
+use vars qw(@ISA);
 @ISA = qw(Locale::Po4a::TransTractor Pod::Parser);
 
 sub initialize {}
 
+sub translate {
+    my ($self,$str,$ref,$type) = @_;
+    my (%options)=@_;
+
+    $str = $self->pre_trans($str,$ref,$type);
+    $str = $self->SUPER::translate($str, $ref, $type, %options);
+    $str = $self->post_trans($str,$ref,$type);
+
+    return $str;
+}
+
+sub pre_trans {
+    my ($self,$str,$ref,$type)=@_;
+
+    return $str;
+}
+
+sub post_trans {
+    my ($self,$str,$ref,$type)=@_;
+
+    # Change ascii non-breaking space to POD one
+    my $nbs_out = "\xA0";
+    my $enc_length = Encode::from_to($nbs_out, "latin1",
+                                               $self->get_out_charset);
+    if (defined $enc_length) {
+        while ($str =~ m/(^|.*\s)(\S+?)\Q$nbs_out\E(\S+?)(\s.*$|$)/s) {
+            my ($begin, $m1, $m2, $end) = ($1, $2, $3, $4);
+            $str  = (defined $begin)?$begin:"";
+            # Remove the non-breaking spaces in the string that will be
+            # between S<...>
+            $m2 =~ s/\Q$nbs_out\E/ /g;
+            $str .= "S<$m1 $m2>";
+            $str .= (defined $end)?$end:"";
+        }
+    }
+
+    return $str;
+}
+
+
 sub command {
     my ($self, $command, $paragraph, $line_num) = @_;
 #    print STDOUT "cmd: '$command' '$paragraph' at $line_num\n";
@@ -45,7 +82,7 @@ sub command {
 				    $self->input_file().":$line_num",
 				    "=$command",
 				    "wrap"=>1);
-	$self->pushline("=$command $paragraph\n");
+	$self->pushline("=$command $paragraph\n\n");
     }
 }
 
@@ -53,6 +90,10 @@ sub verbatim {
     my ($self, $paragraph, $line_num) = @_;
 #    print "verb: '$paragraph' at $line_num\n";
 
+    if ($paragraph eq "\n") {
+	$self->pushline("$paragraph\n");
+	return;
+    }
     $paragraph=$self->translate($paragraph,
 				$self->input_file().":$line_num",
 				"verbatim");
@@ -72,7 +113,11 @@ sub textblock {
     # That way, we'll declare more paragraphs as verbatim than needed, but
     #  that's harmless (only less confortable for translators).
 
-    if ($paragraph =~ m/^ /m) {
+    if ($paragraph eq "\n") {
+	$self->pushline("$paragraph\n");
+	return;
+    }
+    if ($paragraph =~ m/^[ \t]/m) {
 	$self->verbatim($paragraph, $line_num) ;
 	return;
     }
@@ -82,7 +127,7 @@ sub textblock {
 				'textblock',
 				"wrap"=>1);
     $paragraph=~ s/ +\n/\n/gm;
-    $self->pushline("$paragraph\n");
+    $self->pushline("$paragraph\n\n");
 }
 
 sub end_pod {}
@@ -100,6 +145,14 @@ sub parse {
 }
 
 sub docheader {
+    my $self=shift;
+    my $encoding = $self->{TT}{file_out_charset};
+    if (defined $encoding and length $encoding) {
+        $encoding = "\n=encoding $encoding\n";
+    } else {
+        $encoding = "";
+    }
+
     return <<EOT;
 
         *****************************************************
@@ -107,13 +160,12 @@ sub docheader {
         * THIS IS NO SOURCE FILE, BUT RESULT OF COMPILATION *
         *****************************************************
 
-This file was generated by po4a(7) version $VERSION. Do not store
-it (in cvs, for example), but store the po file used as source file by
-pod-translate. 
+This file was generated by po4a(7). Do not store it (in cvs, for example),
+but store the po file used as source file by po4a-translate. 
 
 In fact, consider this as a binary, and the po file as a regular .c file:
 If the po get lost, keeping this translation up-to-date will be harder.
-
+$encoding
 EOT
 }
 1;
@@ -155,7 +207,7 @@ pages, see below) which contains:
 
   CE<lt>" #n"E<gt>
 
-Lake of luck, in the po4a version, this was splited on the space by the
+Lack of luck, in the po4a version, this was splited on the space by the
 wrapping. As result, in the original version, the man contain 
 
  " #n"
@@ -164,10 +216,10 @@ and mine contain
 
  "" #n""
  
-which is logic since C<blabla> is rewritten "blabla"
+which is logic since C<foobar> is rewritten "foobar"
 
 Complete list of pages having this problem on my box (from 564 pages ; note
-that it depend on the choosen wrapping colon):
+that it depends on the chosen wrapping column):
 /usr/lib/perl5/Tk/MainWindow.pod
 /usr/share/perl/5.8.0/overload.pod
 /usr/share/perl/5.8.0/pod/perlapi.pod
@@ -187,7 +239,7 @@ one creates a new parser with C<< Locale::Po4a::Pod->new() >> and then
 calls either parse_from_filehandle() or parse_from_file().
 
 new() can take options, in the form of key/value pairs, that control the
-behavior of the parser.  The recognized options common to all Pod:;Parser
+behavior of the parser.  The recognized options common to all Pod::Parser
 children are:
 
 =over 4
@@ -242,40 +294,21 @@ The column at which to wrap text on the right-hand side.  Defaults to 76.
 
 =back
 
-The options specific to Locale::Po4a::Pod are:
-
-=over 4
-
-=item pomode
-
-In fact, this parser can work in to modes. When pomode=write, it reads a
-pod file, and extract the string which should be translated to a pot
-file. When pomode=read (the default), it reads a pod file and write a
-similar pod file, but translating all strings on the way (the translation
-is read from the po file).
-
-=item dontwrite
-
-if set, no file is written at the end (if you want to manipulate the
-extracted po file in memory, for example).
-
-=back
-
 =head1 SEE ALSO
 
-L<Pod::Parser>, L<po4a(7)>, L<Locale::Po4a::TransTranctor(3pm)>,
+L<Pod::Parser>, L<po4a(7)|po4a.7>, L<Locale::Po4a::TransTractor(3pm)>,
 L<Locale::Po4a::Man(3pm)>,
 
 =head1 AUTHORS
 
  Denis Barbier <barbier@linuxfr.org>
- Martin Quinson <martin.quinson@tuxfamily.org>
+ Martin Quinson (mquinson#debian.org)
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2002 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL (see the COPYING file).
 
 =cut
