@@ -368,17 +368,25 @@ sub parse_file {
         $origfile .= ${$self->{TT}{doc_in}}[$i];
         $i+=2;
     }
+
     unless ($self->{options}{'force'}) {
-    # Detect if we can find the DTD
-    open (VALID, "| nsgmls -p")
-        or die wrap_mod("po4a::sgml",
-                        dgettext("po4a", "Can't run nsgmls -p: %s"), $!);
-    print VALID $origfile;
-    close VALID
-        or die wrap_mod("po4a::sgml",
-                        dgettext("po4a", "Error while running nsgmls -p.  ".
-                                         "Please check if nsgmls and the ".
-                                         "DTD are installed."));
+        # Detect if we can find the DTD
+        my ($tmpfh,$tmpfile)=File::Temp::tempfile("po4a-XXXX",
+                                                  SUFFIX => ".sgml",
+                                                  DIR    => "/tmp",
+                                                  UNLINK => 0);
+        print $tmpfh $origfile;
+        close $tmpfh
+            or die wrap_mod("po4a::sgml",
+                            dgettext("po4a", "Can't close tempfile: %s"), $!);
+        if (system("nsgmls -p < $tmpfile")) {
+            unlink ($tmpfile);
+            die wrap_mod("po4a::sgml",
+                         dgettext("po4a", "Error while running nsgmls -p.  ".
+                                          "Please check if nsgmls and the ".
+                                          "DTD are installed."));
+        }
+        unlink ($tmpfile);
     }
     # Detect the XML pre-prolog
     if ($origfile =~ s/^(\s*<\?xml[^?]*\?>)//) {
@@ -452,20 +460,22 @@ sub parse_file {
 			                    "tag taglist titlepag toc");
 
     } elsif ($prolog =~ /docbook/i) {
-	$self->set_tags_kind("translate" => "abbrev acronym artheader attribution ".
+	$self->set_tags_kind("translate" => "abbrev appendixinfo artheader attribution ".
+	                                    "biblioentry biblioset ".
+	                                    "chapterinfo collab collabname confdates confgroup conftitle ".
 	                                    "date ".
-	                                    "edition editor entry ".
+	                                    "edition editor entry example ".
 	                                    "figure ".
 	                                    "glosssee glossseealso glossterm ".
 	                                    "holder ".
 	                                    "member msgaud msglevel msgorig ".
-	                                    "orgname othername ".
-	                                    "para phrase pubdate publishername primary ".
+	                                    "orgdiv orgname othername ".
+	                                    "pagenums para phrase pubdate publishername primary ".
 	                                    "refclass refdescriptor refentrytitle refmiscinfo refname refpurpose releaseinfo remark revnumber revremark ".
-	                                    "screeninfo seg secondary segtitle simpara subtitle synopfragmentref synopsis ".
+	                                    "screeninfo seg secondary see seealso segtitle simpara substeps subtitle synopfragmentref synopsis ".
 	                                    "term tertiary title titleabbrev ".
 	                                    "contrib epigraph",
-	                     "empty"     => "audiodata colspec graphic imagedata textdata sbr videodata xref",
+	                     "empty"     => "audiodata colspec graphic imagedata textdata sbr spanspec videodata xref",
 	                     "indent"    => "abstract answer appendix article articleinfo audioobject author authorgroup ".
 	                                    "bibliodiv bibliography blockquote blockinfo book bookinfo bridgehead ".
 	                                    "callout calloutlist caption caution chapter copyright ".
@@ -488,26 +498,26 @@ sub parse_file {
 	                                    "warning",
 	                     "verbatim"  => "address cmdsynopsis holder literallayout programlisting ".
 	                                    "refentrytitle refname refpurpose screen term title",
-	                     "ignore"    => "action affiliation anchor application arg author authorinitials ".
-	                                    "command citation citerefentry citetitle classname co computeroutput constant corpauthor ".
+	                     "ignore"    => "acronym action affiliation anchor application arg author authorinitials ".
+	                                    "city citation citerefentry citetitle classname co command computeroutput constant corpauthor country ".
 	                                    "database po4abeg po4aend ".
 	                                    "email emphasis envar errorcode errorname errortext errortype exceptionname ".
 	                                    "filename firstname firstterm footnote footnoteref foreignphrase function ".
 	                                    "glossterm guibutton guiicon guilabel guimenu guimenuitem guisubmenu ".
 	                                    "hardware ".
-	                                    "indexterm informalexample inlineequation inlinegraphic inlinemediaobject interface interfacename ".
+	                                    "indexterm informalexample inlineequation inlinegraphic inlinemediaobject interface interfacename isbn ".
 	                                    "keycap keycode keycombo keysym ".
-	                                    "link literal ".
+	                                    "link lineannotation literal ".
 	                                    "manvolnum markup medialabel menuchoice methodname modespec mousebutton ".
 	                                    "nonterminal ".
 	                                    "olink ooclass ooexception oointerface option optional othercredit ".
-	                                    "parameter personname phrase productname productnumber prompt property ".
+	                                    "parameter personname phrase productname productnumber prompt property pubsnumber ".
 	                                    "quote ".
 	                                    "remark replaceable returnvalue revhistory ".
 	                                    "sgmltag sidebar structfield structname subscript superscript surname symbol systemitem ".
 	                                    "token trademark type ".
 	                                    "ulink userinput ".
-	                                    "varname ".
+	                                    "varname volumenum ".
 	                                    "wordasword ".
 	                                    "xref ".
 	                                    "year",
@@ -707,63 +717,71 @@ sub parse_file {
     }
 
     #   Change the entities including files in the document
-    foreach my $key (keys %entincl) {
-        # The external entity can be referenced as &key; or &key
-        # In the second case, we must differentiate &key and &key2
-        while ($origfile =~/^(.*?)&$key(;.*$|[^-_:.A-Za-z0-9].*$|$)/s) {
-	    my ($begin,$end)=($1,$2);
-	    $end = "" unless (defined $end);
-	    $end =~ s/^;//s;
+    my $dosubstitution = 1;
+    while ($dosubstitution) {
+        $dosubstitution = 0;
+        foreach my $key (keys %entincl) {
+            # The external entity can be referenced as &key; or &key
+            # In the second case, we must differentiate &key and &key2
+            while ($origfile =~/^(.*?)&$key(;.*$|[^-_:.A-Za-z0-9].*$|$)/s) {
+                # Since we will include a new file, we
+                # must do a new round of substitutions.
+                $dosubstitution = 1;
+                my ($begin,$end)=($1,$2);
+                $end = "" unless (defined $end);
+                $end =~ s/^;//s;
 
-	    if ($begin =~ m/.*<!--(.*?)$/s and $1 !~ m/-->/s) {
-		# This entity is commented. Just remove it.
-		$origfile = $begin.$end;
-		next;
-	    }
+                if ($begin =~ m/.*<!--(.*?)$/s and $1 !~ m/-->/s) {
+                    # This entity is commented. Just remove it.
+                    $origfile = $begin.$end;
+                    next;
+                }
 
-	    # add the refs
-	    my $len  = $entincl{$key}{'length'}; # number added by the inclusion
-	    my $pre  = ($begin =~ tr/\n/\n/); # number of \n
-	    my $post = ($end =~ tr/\n/\n/);
-	    print "XX Add a ref. pre=$pre; len=$len; post=$post\n" if $debug{'refs'};
-	    # Keep a reference of inclusion position in main file
-	    my $main = $refs[$pre];
+                # add the refs
+                my $len  = $entincl{$key}{'length'}; # number added by the inclusion
+                my $pre  = ($begin =~ tr/\n/\n/); # number of \n
+                my $post = ($end =~ tr/\n/\n/);
+                print "XX Add a ref. pre=$pre; len=$len; post=$post\n"
+                    if $debug{'refs'};
+                # Keep a reference of inclusion position in main file
+                my $main = $refs[$pre];
 
-	    # Remove the references for the lines after the inclusion
-	    # point.
-	    my @endrefs = splice @refs, $pre+1;
+                # Remove the references for the lines after the inclusion
+                # point.
+                my @endrefs = splice @refs, $pre+1;
 
-	    # Add the references of the added lines
-	    my $i;
-	    for ($i=0; $i<$len; $i++) {
-		$refs[$i+$pre] = "$main $entincl{$key}{'filename'}:".($i+1);
-	    }
+                # Add the references of the added lines
+                my $i;
+                for ($i=0; $i<$len; $i++) {
+                    $refs[$i+$pre] = "$main $entincl{$key}{'filename'}:".($i+1);
+                }
 
-	    if ($begin !~ m/\n[ \t]*$/s) {
-		if ($entincl{$key}{'content'} =~ m/^[ \t]*\n/s) {
-		    # There is nothing in the first line of the included
-		    # file, and something on the line before the inclusion
-		    # The line reference will be more informative like
-		    # this:
-		    $refs[$pre] = $main;
-		}
-	    }
-	    if ($end !~ s/^[ \t]*\n//s) {
-		if ($entincl{$key}{'content'} =~ m/\n[ \t]*$/s) {
-		    # There is something on the line after the inclusion,
-		    # and there is an end of line at the end of the
-		    # included file. We must add the line reference of the
-		    # remainder on the line:
-		    push @refs, $main;
-		}
-	    }
-	    # Append the references removed earlier (lines after the
-	    # inclusion point).
-	    push @refs, @endrefs;
+                if ($begin !~ m/\n[ \t]*$/s) {
+                    if ($entincl{$key}{'content'} =~ m/^[ \t]*\n/s) {
+                        # There is nothing in the first line of the
+                        # included file, and something on the line before
+                        # the inclusion The line reference will be more
+                        # informative like this:
+                        $refs[$pre] = $main;
+                    }
+                }
+                if ($end !~ s/^[ \t]*\n//s) {
+                    if ($entincl{$key}{'content'} =~ m/\n[ \t]*$/s) {
+                        # There is something on the line after the
+                        # inclusion, and there is an end of line at the
+                        # end of the included file. We must add the line
+                        # reference of the remainder on the line:
+                        push @refs, $main;
+                    }
+                }
+                # Append the references removed earlier (lines after the
+                # inclusion point).
+                push @refs, @endrefs;
 
-	    # Do the substitution
-	    $origfile = "$begin".$entincl{$key}{'content'}."$end";
-	    print STDERR "substitute $2\n" if ($debug{'entities'});
+                # Do the substitution
+                $origfile = "$begin".$entincl{$key}{'content'}."$end";
+                print STDERR "substitute $key\n" if ($debug{'entities'});
+            }
         }
     }
     $origfile=~s/\G(.*?)&([A-Za-z_:][-_:.A-Za-z0-9]*|#[0-9]+|#x[0-9a-fA-F]+)\b/$1\{PO4A-amp\}$2/gs;
