@@ -469,7 +469,7 @@ sub get_leading_command {
     print STDERR "get_leading_command($buffer)="
         if ($debug{'extract_commands'});
 
-    if ($buffer =~ m/^$RE_ESCAPE([[:alpha:]]+)(\*?)(.*)$/s
+    if ($buffer =~ m/^$RE_ESCAPE([[:alnum:]]+)(\*?)(.*)$/s
         && defined $separated_command{$1}) {
         # The buffer begin by a comand (possibly preceded by some
         # whitespaces).
@@ -477,7 +477,7 @@ sub get_leading_command {
         $variant = $2;
         $buffer  = $3;
         # read the arguments (if any)
-        while ($buffer =~ m/^\s*([\[\{])(.*)$/s) {
+        while ($buffer =~ m/^\s*$RE_PRE_COMMENT([\[\{])(.*)$/s) {
             my $type = $1;
             my $arg = "";
             my $count = 1;
@@ -485,7 +485,7 @@ sub get_leading_command {
             # stop reading the buffer when the number of ] (or }) matches the
             # the number of [ (or {).
             while ($count > 0) {
-                if ($buffer =~ m/^(.*?)([\[\]\{\}])(.*)$/s) {
+                if ($buffer =~ m/^(.*?$RE_PRE_COMMENT)([\[\]\{\}])(.*)$/s) {
                     $arg .= $1;
                     $buffer = $3;
                     if ($2 eq $type) {
@@ -556,8 +556,8 @@ sub get_trailing_command {
 
     # While the buffer ends by }, consider it is a mandatory argument
     # and extract this argument.
-    while (   $buffer =~ m/^(.*(\{).*)\}$/s
-           or $buffer =~ m/^(.*(\[).*)\]$/s) {
+    while (   $buffer =~ m/^(.*$RE_PRE_COMMENT(\{).*)$RE_PRE_COMMENT\}$/s
+           or $buffer =~ m/^(.*$RE_PRE_COMMENT(\[).*)$RE_PRE_COMMENT\]$/s) {
         my $arg = "";
         my $count = 1;
         $buffer = $1;
@@ -565,7 +565,7 @@ sub get_trailing_command {
         # stop reading the buffer when the number of } (or ]) matches the
         # the number of { (or [).
         while ($count > 0) {
-            if ($buffer =~ m/^(.*)([\{\}\[\]])(.*)$/s) {
+            if ($buffer =~ m/^(.*$RE_PRE_COMMENT)([\{\}\[\]])(.*)$/s) {
                  $arg = $3.$arg;
                  $buffer = $1;
                  if ($2 eq $type) {
@@ -588,7 +588,7 @@ sub get_trailing_command {
     }
 
     # There should now be a command, maybe followed by an asterisk.
-    if ($buffer =~ m/^(.*)$RE_ESCAPE([[:alpha:]]+)(\*?)\s*$/s
+    if ($buffer =~ m/^(.*$RE_PRE_COMMENT)$RE_ESCAPE([[:alnum:]]+)(\*?)\s*$/s
         && defined $separated_command{$2}) {
         $buffer = $1;
         $command = $2;
@@ -738,7 +738,7 @@ sub translate_buffer {
         if ($buffer =~ /^(\s+)(.*?)$/s) {
             $spaces = $1;
 #            $buffer = $2; # FIXME: this also remove trailing spaces!!
-            $buffer =~ s/^\s*//;
+            $buffer =~ s/^\s*//s;
         }
         my $buffer_save = $buffer;
         ($command, $variant, $args, $buffer) =
@@ -767,7 +767,7 @@ sub translate_buffer {
                 if ($buffer =~ /^(\s+)(.*?)$/s) {
                     $spaces = $1;
 #                    $buffer = $2;  # FIXME: this also remove trailing spaces!!
-                    $buffer =~ s/^\s*//;
+                    $buffer =~ s/^\s*//s;
                 }
                 $translated_buffer .= $spaces;
             } else {
@@ -846,16 +846,19 @@ sub translate_buffer {
         }
         $wrap = 0 if (defined $no_wrap and $no_wrap == 1);
         # Keep spaces at the end of the buffer.
-        my $spaces = "";
+        my $spaces_end = "";
         if ($buffer =~ /^(.*?)(\s+)$/s) {
-            $spaces = $2;
+            $spaces_end = $2;
             $buffer = $1;
         }
+        if ($wrap and $buffer =~ s/^(\s+)//s) {
+                $translated_buffer .= $1;
+        }
         $translated_buffer .= $self->translate($buffer,$self->{ref},
                                                @env?$env[-1]:"Plain text",
                                                "wrap" => $wrap);
         # Restore spaces at the end of the buffer.
-        $translated_buffer .= $spaces;
+        $translated_buffer .= $spaces_end;
     }
 
     # append the translation of the trailing commands
@@ -904,8 +907,8 @@ sub read {
 =item read_file
 
 Recursively read a file, appending included files which are not listed in the
-@exclude_include array.  Included files are searched in the directory of the
-input document or in a directory listed in the TEXINPUTS environment variable.
+@exclude_include array.  Included files are searched using the B<kpsewhich>
+command from the Kpathsea library.
 
 Except from the file inclusion part, it is a cut and paste from
 Transtractor's read.
@@ -952,18 +955,9 @@ sub read_file {
             }
             if ($include) {
                 # search the file
-                foreach (($my_dirname,
-                          defined($ENV{"TEXINPUTS"})?
-                              split(/:/, $ENV{"TEXINPUTS"}):
-                              ".")) {
-                    if      (-r "$_/$newfilename") {
-                        $newfilename = "$_/$newfilename";
-                        last;
-                    } elsif (-r "$_/$newfilename.tex") {
-                        $newfilename = "$_/$newfilename.tex";
-                        last;
-                    }
-                }
+                open (KPSEA, "kpsewhich " . $newfilename . " |");
+                $newfilename = <KPSEA>;
+
                 push @entries, read_file($self,
                                          $newfilename);
                 if ($tag eq "include") {
@@ -1011,17 +1005,18 @@ newcommands).
 
 sub parse_definition_file {
     my ($self,$filename,$only_try)=@_;
+    my $filename_org = $filename;
 
-    foreach (($my_dirname,
-              defined($ENV{"TEXINPUTS"})?
-                  split(/:/, $ENV{"TEXINPUTS"}):
-                  ".")) {
-        if ((not defined $_) or (not length $_)) {
-            $_ = ".";
-        }
-        if (-r $_."/".$filename) {
-            $filename = $_."/".$filename;
-            last;
+    open (KPSEA, "kpsewhich " . $filename . " |");
+    $filename = <KPSEA>;
+
+    if (not defined $filename) {
+        warn wrap_mod("po4a::tex",
+            dgettext("po4a", "kpsewhich cannot find %s"), $filename_org);
+        if (defined $only_try && $only_try) {
+            return;
+        } else {
+            exit 1;
         }
     }
 
@@ -1084,9 +1079,9 @@ sub parse_definition_line {
         if ($line =~ m/^\s*((?:\{_?\}|\[_?\])*)\s*$/) {
             register_generic_environment("$env,$1");
         }
-    } elsif ($line =~ /^separator\s+(\w+(?:\[#[0-9]+\]))\s+\"(.*)\"\s*$/) {
+    } elsif ($line =~ /^separator\s+(\w+(?:\[#[0-9]+\])?)\s+\"(.*)\"\s*$/) {
         my $env = $1; # This is not necessarily an environment.
-                      # It can also be smth like 'title{#1}'.
+                      # It can also be smth like 'title[#1]'.
         $env_separators{$env} = $2;
     } elsif ($line =~ /^verbatim\s+environment\s+(\w+)\s+$/) {
         register_verbatim_environment($1);
@@ -1105,15 +1100,15 @@ sub is_closed {
     my $opening = 0;
     # FIXME: { and } should not be counted in verbatim blocks
     # Remove comments
-    $tmp =~ s/($RE_PRE_COMMENT)$RE_COMMENT.*//mg;
-    while ($tmp =~ /^.*?(?<!$RE_ESCAPE)(?:$RE_ESCAPE$RE_ESCAPE)*\{(.*)$/s) {
+    $tmp =~ s/$RE_PRE_COMMENT$RE_COMMENT.*//mg;
+    while ($tmp =~ /^.*?$RE_PRE_COMMENT\{(.*)$/s) {
         $opening += 1;
         $tmp = $1;
     }
     $tmp = $paragraph;
     # Remove comments
-    $tmp =~ s/($RE_PRE_COMMENT)$RE_COMMENT.*//mg;
-    while ($tmp =~ /^.*?(?<!$RE_ESCAPE)(?:$RE_ESCAPE$RE_ESCAPE)*\}(.*)$/s) {
+    $tmp =~ s/$RE_PRE_COMMENT$RE_COMMENT.*//mg;
+    while ($tmp =~ /^.*?$RE_PRE_COMMENT\}(.*)$/s) {
         $closing += 1;
         $tmp = $1;
     }
@@ -1384,7 +1379,7 @@ TEST_TYPE:
         }
         @e = @$env;
         my $wrap = 1;
-        $wrap = 0 if $no_wrap == 1;
+        $wrap = 0 if (defined $no_wrap and $no_wrap == 1);
         $translated = $self->translate($tmp,$self->{ref},
                                        @e?$e[-1]:"Plain text",
                                        "wrap" => $wrap);
