@@ -75,14 +75,12 @@ $VERSION= $Locale::Po4a::TeX::VERSION;
 
 use Locale::Po4a::Common;
 use Locale::Po4a::TeX;
-use subs qw(&untranslated     &translate_joined
+use subs qw(
             &parse_definition_file
             &register_generic_command &is_closed &translate_buffer
             &register_verbatim_environment
             &generic_command
             &in_verbatim);
-*untranslated                  = \&Locale::Po4a::TeX::untranslated;
-*translate_joined              = \&Locale::Po4a::TeX::translate_joined;
 *parse_definition_file         = \&Locale::Po4a::TeX::parse_definition_file;
 *register_generic_command      = \&Locale::Po4a::TeX::register_generic_command;
 *register_verbatim_environment = \&Locale::Po4a::TeX::register_verbatim_environment;
@@ -120,6 +118,7 @@ $ESCAPE = "\@";
 $RE_ESCAPE = "\@";
 $RE_VERBATIM = "\@example";
 $RE_COMMENT = "\\\@(?:c|comment)\\b";
+$RE_PRE_COMMENT = "(?<!\@)(?:\@\@)*";
 
 my %break_line = ();
 
@@ -343,7 +342,7 @@ sub translate_buffer_ignore {
 }
 $translate_buffer_env{"ignore"} = \&translate_buffer_ignore;
 
-foreach (qw(appendix section cindex findex kindex pindex vindex subsection
+foreach (qw(appendix section cindex findex kindex opindex pindex vindex subsection
             dircategory subtitle include
             exdent center unnumberedsec
             heading unnumbered unnumberedsubsec
@@ -393,9 +392,9 @@ foreach (qw(defcvx deffnx defivarx defmacx defmethodx defopx defoptx
 foreach (qw(titlefont w i r b sansserif sc slanted strong t cite email
             footnote indicateurl emph ref xref pxref inforef kbd key
             acronym),
-# The following commands could cause problems since the their arguments
-# has a semantic and a translator could decide not to translate code but
-# still translate thses short words if they appear in another context.
+# The following commands could cause problems since their arguments
+# have a semantic and a translator could decide not to translate code but
+# still translate theses short words if they appear in another context.
          qw(file command dfn dmn option math code samp var)) {
     register_generic_command("-$_,{_}");
 }
