@@ -11,7 +11,7 @@ use warnings;
 
 use subs qw(makespace);
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION="0.33.3";
+$VERSION="0.36.1";
 @ISA = qw(DynaLoader);
 @EXPORT = qw(new process translate 
              read write readpo writepo
@@ -503,8 +503,6 @@ sub stats   {
     return $_[0]->{TT}{po_in}->stats_get(); 
 }
 
-=cut
-
 =head2 Manipulating addenda
 
 =over 4
@@ -725,7 +723,10 @@ sub shiftline   {
 		     shift @{$_[0]->{TT}{doc_in}}); 
     return ($line,$ref);
 }
-sub unshiftline {  unshift @{$_[0]->{TT}{doc_in}},($_[1],$_[2]);  }
+sub unshiftline {
+	my $self = shift;
+	unshift @{$self->{TT}{doc_in}},@_;
+}
 
 sub pushline    {  push @{$_[0]->{TT}{doc_out}}, $_[1] if defined $_[1]; }
 sub popline     {  return pop @{$_[0]->{TT}{doc_out}};            }
@@ -894,6 +895,14 @@ sub translate {
 #	Encode::from_to($transstring,$self->{TT}{po_in}->get_charset,
 #	    $self->get_out_charset);
 #    }
+
+    if ($options{'wrap'}||0) {
+        $transstring =~ s/( *)$//s;
+        my $trailing_spaces = $1||"";
+        $transstring =~ s/ *$//gm;
+        $transstring .= $trailing_spaces;
+    }
+
     return $transstring;
 }
 
